<?php

namespace Drupal\codit_batch_operations_ui\Controller;

use Drupal\codit_batch_operations\Entity\BatchOpLog;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\PluralTranslatableMarkup;
use Drupal\Core\Utility\TableSort;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Class Operations Controller.
 *
 * @package Drupal\codit_batch_operations_ui\Controller
 */
class OperationPage extends OperationsBase {

  /**
   * Renders the batch operation page based on the parameter in the URI.
   *
   * @param string $batch_operation_name
   *   The class name of the batch operation from URI parameter.
   *
   * @return array
   *   A render array for the batch operation page.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   In the case where the parameter does not match an existing BatchOp class.
   */
  public function renderOperation(string $batch_operation_name): array {
    if (!$this->isBatchOperation($batch_operation_name)) {
      // Does not exist, return 404.
      throw new NotFoundHttpException();
    }
    $page = [];
    $page['page'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [
            'batch-operation',
        ],
    ],
    ];
    $batch_op_class_name = $this->getClassUri($batch_operation_name);
    $batch_operation = $this->getBatchOperationClass($batch_operation_name);

    $page['page']['title'] = [
      '#type' => 'html_tag',
      '#tag' => 'h2',
      '#value' => $batch_operation->getTitle(),
    ];
    $description = $batch_operation->getDescription();
    if (!empty($description)) {
      $page['page']['description'] = [
            'label' => [
              '#type' => 'label',
              '#title' => $this->t('Description'),
              '#title_display' => 'above',
            ],
            'value' => [
              '#type' => 'container',
              '#markup' => "<p class=\"description\">{$batch_operation->getDescription()}</p>",
            ],
      ];
    }

    $batch_size = number_format($batch_operation->getBatchSize());
    $batch_size_text = new PluralTranslatableMarkup(
        $batch_size,
        '1 item at a time',
        '@count items at a time'
      );
    $label = $this->t('Batch size:');
    $page['page']['batch_size'] = [
      '#type' => 'html_tag',
      '#tag' => 'p',
      '#value' => "<span class=\"item__label\">$label</span> <span>$batch_size_text</span>",
    ];

    $item_count = number_format(count($batch_operation->gatherItemsToProcess()));
    $item_count_text = new PluralTranslatableMarkup(
        $item_count,
        '1 item',
        '@count items '
      );
    $label = $this->t('Total items to process:');
    $page['page']['item_count'] = [
      '#type' => 'html_tag',
      '#tag' => 'p',
      '#value' => "<span class=\"item__label\">$label</span> <span>$item_count_text</span>",
    ];
    $page['breaker'] = [
      '#markup' => '<hr>',
    ];

    $page['form'] = $this->formBuilder->getForm('\Drupal\codit_batch_operations_ui\Form\BatchOperationRun', $this->getClassUri($batch_operation_name), $batch_operation->getTitle());

    $page['table-breaker'] = [
      '#markup' => '<hr>',
    ];
    $header = [
      ['data' => $this->t('Run date'), 'field' => 'run-date', 'sort' => 'desc'],
      ['data' => $this->t('Duration'), 'field' => 'duration'],
      ['data' => $this->t('Run method'), 'field' => 'run-method'],
      ['data' => $this->t('Run by'), 'field' => 'run-by'],
      ['data' => $this->t('Status'), 'field' => 'status'],
      ['data' => $this->t('Operation'), 'field' => 'operation'],
    ];
    $rows = $this->assembleRows($batch_op_class_name, $batch_size);
    $order = TableSort::getOrder($header, $this->request);
    $direction = TableSort::getSort($header, $this->request);
    $rows = $this->sortRows($rows, $order['sql'], $direction);
    $rows = $this->convertDuration($rows);
    $rows = $this->convertDate($rows);
    $page['log_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#caption' => $this->t('Logs of previous runs.'),
      '#rows' => $rows,
      '#empty' => $this->t('This has no previous logs.'),
    ];
    $page['pager'] = [
      '#type' => 'pager',
    ];
    return $page;
  }

  /**
   * Renders the batch operation page based on the parameter in the URI.
   *
   * @param string $batch_operation_name
   *   The class name of the batch operation from URI parameter.
   * @param string $skip
   *   Whether should 'fail' or 'allow_skip'.
   *
   * @return array
   *   A render array for the batch operation page.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   In the case where the parameter does not match an existing BatchOp class.
   */
  public function renderOperationConfirmation(string $batch_operation_name, string $skip): array {
    if (!$this->isBatchOperation($batch_operation_name) || !$this->isValidSkipOption($skip)) {
      // Does not exist, return 404.
      throw new NotFoundHttpException();
    }
    $form = $this->formBuilder->getForm('\Drupal\codit_batch_operations_ui\Form\BatchOperationRunConfirmation', $this->getClassUri($batch_operation_name), $skip);

    return $form;
  }

  /**
   * Checks to see if the arg is valid.
   *
   * @param string $skip_arg
   *   The arg to check.
   *
   * @return bool
   *   TRUE if the arg is valid, FALSE otherwise.
   */
  protected function isValidSkipOption(string $skip_arg) : bool {
    $valid = ['skip', 'fail'];
    return (in_array($skip_arg, $valid)) ? TRUE : FALSE;
  }

  /**
   * Page title callback.
   *
   * @param string $batch_operation_name
   *   The name of the operation, from the URI parameter.
   *
   * @return string
   *   The fully translated title.
   */
  public function getTitle(string $batch_operation_name): string {
    $vars = [
      '@name' => $batch_operation_name,
    ];
    return $this->t('Operation: @name', $vars);
  }

  /**
   * Assemble the list rows, one per BatchOpLog.
   *
   * @param string $batch_operation_name
   *   The class name of a batch operation.
   * @param ?int $batch_size
   *   The current size of the batch.
   *
   * @return array
   *   An array of rows.
   */
  protected function assembleRows(string $batch_operation_name, $batch_size): array {
    $rows = [];
    $batchOpLog = new BatchOpLog([], 'batch_op_log');
    $batch_op_logs = $batchOpLog->getRelatedBatchOpLogs(trim($batch_operation_name, '\\'));

    foreach ($batch_op_logs as $batch_op_log) {
      $row = [
        'run-date' => ($batch_op_log) ? $batch_op_log->get('last')->value : '-',
        'duration' => $batch_op_log->get('last')->value - $batch_op_log->get('created')->value,
        'run-method' => ($batch_op_log) ? $batch_op_log->get('executor')->value : '-',
        'run-by' => $batch_op_log->get('user_id')->entity->toLink($batch_op_log->get('user_id')->entity->get('name')->value),
        'status' => $this->buildCompleteStatus($batch_op_log, $batch_size),
        'operation' => new Link($this->t('View log'), $batch_op_log->getUrl()),
      ];
      $rows[] = $row;
    }

    return $rows;
  }

}
