<?php

namespace Drupal\codit_batch_operations\Drush\Commands;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Utility\Token;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;
use Drush\Drush;
use Drush\Exceptions\CommandFailedException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A Drush command file.
 */
final class CoditBatchOperationsCommands extends DrushCommands implements ContainerInjectionInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a ContentModelDocumentationCommands object.
   */
  public function __construct(
    private readonly Token $token,
    ConfigFactoryInterface $config_factory,
  ) {
    parent::__construct();
    $this->configFactory = $config_factory;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('token'),
      $container->get('config.factory')
    );
  }

  /**
   * Runs the BatchOp script if it exists.
   *
   * @param string $script
   *   The name of the BatchOp script to run.
   * @param array $options
   *   The array of option flags passed in.
   */
  #[CLI\Command(name: 'codit-batch-operations:run', aliases: ['batch-op-run'])]
  #[CLI\Argument(name: 'script', description: 'The Batch Operation script to run.')]
  #[CLI\Option(name: 'allow-skip', description: 'Allow skipping batch items if they error.')]
  #[CLI\Usage(name: 'drush codit-batch-operations:run TestDo10Things', description: 'Runs the script named TestDo10Things.')]
  public function run(string $script, $options = ['allow-skip' => FALSE]) {
    try {
      $scriptClass = $this->prependPath($script);
      $this->checkScriptClass($scriptClass);
      $sandbox = [];
      // @phpstan-ignore-next-line
      $script = \Drupal::classResolver($scriptClass);

      do {
        $msg = $script->run($sandbox, 'drush', $options['allow-skip']);
        Drush::output()->writeln($msg);
      } while ($sandbox['#finished'] < 1);
      if ($sandbox['skipped_items']) {
        Drush::output()->writeln('Skipped items:');
        Drush::output()->writeln(print_r($sandbox['skipped_items'], TRUE));

      }
      if ($script->getLoggedErrors()) {
        Drush::output()->writeln('Errors:');
        Drush::output()->writeln(print_r($script->getLoggedErrors(), TRUE));
      }
      $msg = '';
    }
    catch (\Exception $e) {
      $vars = [
        '@error' => $e->getMessage(),
      ];
      $msg = dt("Codit Batch Operations:  @error", $vars);
    }

    return $msg;
  }

  /**
   * Add the namespaced path to the filename.
   *
   * @param string $script
   *   The filename of the script without the file extension.
   *
   * @return string
   *   The namespaced path to the class.
   */
  protected function prependPath($script): string {
    $test_scripts = [
      'TestDo10Things',
      'TestDo10ThingsWithError',
      'TestDo10000Things',
    ];
    if (in_array($script, $test_scripts)) {
      // This is a test script, so route it to codit_batch_operations.
      return "\Drupal\codit_batch_operations\cbo_scripts\\$script";
    }
    else {
      $config = $this->configFactory->getEditable('codit_batch_operations.settings');
      $module = $config->get('script_location');
      return "\Drupal\\$module\cbo_scripts\\$script";
    }

  }

  /**
   * Check to see if the script exists and is the right interface.
   *
   * @param string $scriptClass
   *   The fully namespaced class name of the script.
   *
   * @throws \Drush\Exceptions\CommandFailedException
   *   If the script is not available or not the right interface.
   */
  protected function checkScriptClass($scriptClass): void {
    if (!class_exists($scriptClass)) {
      throw new CommandFailedException("The class $scriptClass does not exist.");
    }

    $implementations = class_implements($scriptClass, TRUE);
    $required_interface = 'Drupal\codit_batch_operations\BatchScriptInterface';
    if (empty($implementations) || !in_array($required_interface, $implementations)) {
      // The class is not the right interface.
      throw new CommandFailedException("The class $scriptClass must implement $required_interface.");
    }
  }

}
